/*
 * @(#)TextChaToolBar.java
 *
 * Copyright (c) 2009 by the original authors of AnyWikiDraw
 * and all its contributors.
 * All rights reserved.
 *
 * The copyright of this software is owned by the authors and  
 * contributors of the AnyWikiDraw project ("the copyright holders").
 * You may not use, copy or modify this software, except in  
 * accordance with the license agreement you entered into with  
 * the copyright holders. For details see accompanying license terms. 
 */
package org.anywikidraw.any;

import org.jhotdraw.util.*;

import java.awt.*;
import javax.swing.*;

/**
 * Displays a textual challenge question, which must be responded to.
 * <p>
 * The challenge is sent by the server. If the answer given by the user is
 * not correct, the server refuses to save the drawing.
 * <p>
 * Textual challenge questions are a measure to prevent spam.
 *
 * @author Werner Randelshofer
 * @version $Id: TextChaToolBar.java 135 2009-10-06 05:15:05Z rawcoder $
 */
public class TextChaToolBar extends JToolBar {

    private ResourceBundleUtil labels;

    /** Creates new instance. */
    public TextChaToolBar() {
        labels = ResourceBundleUtil.getBundle("org.jhotdraw.samples.svg.Labels");
        initComponents();

/*        GridBagLayout layout = new GridBagLayout();
        setLayout(layout);

        GridBagConstraints gbc;
        gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.BASELINE_LEADING;
        gbc.weighty = 1d;
        gbc.fill = GridBagConstraints.BOTH;
        layout.setConstraints(questionLabel, gbc);

        gbc = new GridBagConstraints();
        gbc.insets = new Insets(0, 10, 0, 0);
        gbc.weightx = 1d;
        gbc.fill = GridBagConstraints.NONE;
        gbc.anchor = GridBagConstraints.BASELINE_LEADING;
        layout.setConstraints(answerField, gbc);
        //questionLabel.setMinimumSize(new Dimension(200,20));*/
        answerField.setMinimumSize(answerField.getPreferredSize());
        answerField.setMaximumSize(answerField.getPreferredSize());
    }

    public void setQuestion(String newValue) {
        questionLabel.setText("<html>"+AbstractDrawingApplet.htmlencode(newValue));
        questionLabel.setMaximumSize(new Dimension(questionLabel.getPreferredSize().width,Integer.MAX_VALUE));
    }

    public String getAnswer() {
        return answerField.getText();
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        questionLabel = new javax.swing.JLabel();
        spacerPanel = new javax.swing.JPanel();
        answerField = new javax.swing.JTextField();

        setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        setFloatable(false);
        setOpaque(false);

        questionLabel.setFont(new java.awt.Font("Dialog", 0, 11)); // NOI18N
        add(questionLabel);

        spacerPanel.setMaximumSize(new java.awt.Dimension(10, 10));
        spacerPanel.setMinimumSize(new Dimension(10,10));
        spacerPanel.setOpaque(false);
        spacerPanel.setPreferredSize(new java.awt.Dimension(10, 10));
        spacerPanel.setLayout(new java.awt.BorderLayout());
        add(spacerPanel);

        answerField.setColumns(10);
        answerField.setFont(new java.awt.Font("DialogInput", 0, 11)); // NOI18N
        add(answerField);
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField answerField;
    private javax.swing.JLabel questionLabel;
    private javax.swing.JPanel spacerPanel;
    // End of variables declaration//GEN-END:variables
}
